/* CXA3235 commands
 */

#include <stdio.h>
#ifdef	MSDOS
#include <io.h>
#include <dos.h>
#endif
#include <fcntl.h>
#include <sys/stat.h>
#include <string.h>
#include "global.h"
#include "i2c.h"
#include "vars.h"
#include "cxa3235.h"
#if	1
#include "msp3417.h"
#endif

#define	DISPSIZE	256

extern	int	i2c_verbose;
extern	int	ifd;

/* Global variables */
 
/* Calculate D1, D2, C and SW bytes
 * Mode = B4
 * Fref = 7.8125 KHz, Step = 50KHz ( /640 )
 * Fosc = Fref x 8 x (32 x M + S)
 * ==>
 * 32 x M + S = Fosc / (Fref x 8)
 */
#define	STEP	50
#define	STEP2	100

void
fset3235(unsigned long freq)
{
	unsigned long fvco,n,m,s;
	unsigned cbyte;
	unsigned char data[4];

	frx = freq;
	if (freq < 210000) {
		cbyte = 0x01;
	} else if (freq < 400000) {
		cbyte = 0x02;
	} else {
		cbyte = 0x08;
	}
	fvco = freq + if_freq;	/* in KHz */
#if	0
	/* cannot divide integer bij 62.5 */
	n = (fvco * 2) / 125;
#endif
#if	0
	/* cannot divide integer bij 31.25 */
	n = (fvco * 4) / 125;
#endif
#if	1
	n = fvco / 50;
#endif
	m = n / 32;
	s = n % 32;
	printf("Fin=%d.%03d MHz, Fvco=%d.%03d MHz, n=0x%x, m=0x%x, s=0x%x\n",
		freq/1000,freq%1000,
		fvco/1000,fvco%1000,
		n,
		m,
		s
		);
	data[0] = (m >> 3) & 0xff;			/* D1 */
	data[1] = ((m & 0x07) << 5) | (s & 0x1f);	/* D2 */
	data[2] = 0xc0;   /* R1=0 R0=0, /640 */		/* C */
	data[3] = cbyte;				/* SW */
	printf("%02x %02x %02x %02x\n",data[0],data[1],data[2],data[3]);

	/* Fosc = Fref x 8 x (32 x M + S)
	 *      = (32 x m + s) * 50.0
	 */
	fvco = (32 * m + s) * 50;
	printf("Fvco=%d.%03d MHz\n",fvco/1000,fvco%1000);

	printf("Loading to %02x: %02x %02x %02x %02x\n",
			ADDR_CXA3235,data[0],data[1],data[2],data[3]);

	if (i2c_write(ifd,ADDR_CXA3235,data,4)) {
		i2c_perror("write");
	}
}

int
dorxset(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	unsigned long freq;

	if (argc != 2) {
		printf("Usage: rxset freq\n");
		return 1;
	}
	freq = atol(argv[1]);
	fset3235(freq);

	return 0;
}

int
docalc3235(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	unsigned long freq,fvco,n,m,s;
	unsigned cbyte;

	if (argc != 2) {
		printf("Usage: calc3235 freq (KHz)\n");
		return(1);
	}
	freq = atol(argv[1]);	/* in KHz */
	if (freq < 110000) {
		cbyte = 0x01;
	} else if (freq < 400000) {
		cbyte = 0x02;
	} else {
		cbyte = 0x08;
	}
	fvco = freq + if_freq;	/* in KHz */
#if	0
	/* cannot divide integer bij 62.5 */
	n = (fvco * 2) / 125;
#endif
#if	0
	/* cannot divide integer bij 31.25 */
	n = (fvco * 4) / 125;
#endif
#if	1
	n = fvco / 50;
#endif
	m = n / 32;
	s = n % 32;
	printf("Fvco=%d.%03d MHz, n=0x%x, m=0x%x, s=0x%x\n",
		fvco/1000,fvco%1000,
		n,
		m,
		s
		);
	printf("%02x %02x %02x %02x\n",
			(m >> 3) & 0xff,
			((m & 0x07) << 5) | (s & 0x1f),
			0xc0,	/* R1=0 R0=1, /640 */
			cbyte
		);
	/* Fosc = Fref x 8 x (32 x M + S)
	 *      = (32 x m + s) * 50.0
	 */
	fvco = (32 * m + s) * 50;
	printf("Fvco=%d.%03d MHz\n",fvco/1000,fvco%1000);
}

/* Read CXA3235 status */
int
dostat3235(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	int r;
	u_char st;

	if ((r = i2c_read(ifd,ADDR_CXA3235+1,&st,1)) != 0) {
		printf("r=%d ",r);
		i2c_perror("");
        }
	printf("status=0x%02x, PR=%c, FL=%c\n",
			st,
			st&0x80?'1':'0',
			st&0x40?'1':'0'
		);
	return 0;
}

/* Load C1, C2, PD1 and PD2 */
int
dold3235(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	int fd,len;
	unsigned char address,data[4];

	if (argc != 5) {
		printf("Usage: ld3235 d1 d2 c sw\n");
		return(1);
	}

	address = ADDR_CXA3235;
	data[0] = htoi(argv[1]);	/* D1 */
	data[1] = htoi(argv[2]);	/* D2 */
	data[2] = htoi(argv[3]);	/* C */
	data[3] = htoi(argv[4]);	/* SW */
	printf("Loading to %02x: %02x %02x %02x %02x\n",
			address,data[0],data[1],data[2],data[3]);

	if (i2c_write(ifd,ADDR_CXA3235,data,4)) {
		i2c_perror("write");
		return(1);
	}

	return(0);
}

/* RX scan */
int
dorxscan(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	unsigned long fmin2,fmax2,step;
	unsigned long freq2,n,m,s,f1;
	unsigned char data[16],st;
	int r;
	unsigned short val_fm1,val_fm2;

	if (argc != 4) {
		printf("Usage: rxscan fmin (KHz) fmax (KHz) step (/50 KHz)\n");
		return 1;
	}

	fmin2 = (atol(argv[1]) + if_freq) * 2;
	fmax2 = (atol(argv[2]) + if_freq) * 2;
	step = atol(argv[3]);
	for (freq2 = fmin2 ; freq2 <= fmax2 ; freq2 += step*STEP2 ) {
		n = freq2 / STEP2;
		m = n / 32;
		s = n % 32;
		f1 = ((32 * m + s) * STEP2) / 2;
		f1 -= if_freq;
		printf("Freq=%d.%03d MHz ",f1/1000,f1%1000);

		data[0] = (m >> 3) & 0xff;
		data[1] = ((m & 0x07) << 5) | (s & 0x1f);
		data[2] = 0xc0;
		if (f1 < 110000) {
			data[3] = 0x01;
		} else if (f1 < 400000) {
			data[3] = 0x02;
		} else {
			data[3] = 0x08;
		}
		printf("%02x %02x %02x %02x ",data[0],data[1],data[2],data[3]);
		if (i2c_write(ifd,ADDR_CXA3235,data,4)) {
			i2c_perror("write");
			return(1);
		}
		usleep(500000);

		if ((r = i2c_read(ifd,ADDR_CXA3235+1,&st,1)) != 0) {
			printf("r=%d ",r);
			i2c_perror("");
		}
		printf("FL=%c", st&0x40?'1':'0');
		usleep(500000);
#if	1
		/* Read MSP3414D DC level registers */
		(void)read_msp3417_register(ADDR_MSP3417,RD_DSP,
			0x001b,&val_fm1);
		(void)read_msp3417_register(ADDR_MSP3417,RD_DSP,
			0x001c,&val_fm2);
		printf(" DC_FM1=0x%04x DC_FM2=0x%04x\n",val_fm1,val_fm2);
#endif
	}

	return 0;
}

/* RX up 1 step */
int
doup(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	if (argc != 1) {
		printf("Usage: up\n");
		return 1;
	}
	fset3235(frx + fstep);

	return 0;
}

