/* I2C interface routines
 */

#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <sys/types.h>
#include <dev/iicbus/iic.h>
#include "i2c.h"

extern	int	errno;

int	i2cfd = -1;			/* I2C filedescriptor		*/
int	i2c_error = 0;			/* Global I2C error code	*/
int	i2c_verbose = 0;		/* Enable debugging info	*/

char	*i2c_error_tab[] = {
	"OK",				/* I2C_OK			*/
	"I2C error",			/* I2C_ERR			*/
	"No such slave",		/* I2C_NOENT			*/
};

#define	NI2CERROR	(sizeof(i2c_error_tab)/sizeof(char *))

extern void printbuf(unsigned char *, int);

int
i2c_init(char *iicdev)
{
	int fd;

	if ((fd = open(iicdev,O_RDWR)) < 0) {
		i2c_error = I2C_ERR;
		return(fd);
	}
	i2cfd = fd;
	if(i2c_verbose)
		printf("i2c_init: fd=%d\n",fd);
	(void)i2c_reset(fd);
	return fd;
}

void
i2c_uninit(int ifd)
{
	close(ifd);
}

#if	1
int
i2c_reset(int fd)
{
	return(0);
}
#else
/* ioctl/I2CRSTCARD doesn't work for lpbb device ?
 */
int
i2c_reset(int fd)
{
	struct iiccmd i;

	i.slave = 0;
	i.count = 0;
	i.last = 0;
	i.buf = NULL;
	if ((ioctl(fd,I2CRSTCARD,&i)) < 0) {
		perror("I2CRSTCARD");
		i2c_error = I2C_ERR;
		return(1);
	}
	return(0);
}
#endif

int
i2c_write(int ifd, unsigned char a, unsigned char *buf, int cnt)
{
	struct iiccmd i;

	if(i2c_verbose) {
		printf("i2c_write: ifd=%d cnt=%d a=%02x  ",ifd,cnt,a);
		if (cnt > 0) {
			printf("data=");
			printbuf(buf,cnt);
		}
		printf("\n");
	}

	i.slave = a;
	i.count = 0;
	i.last = 0;
	i.buf = NULL;
	if ((ioctl(ifd,I2CSTART,&i)) < 0) {
		i2c_error = errno;
		goto error;
	}

	i.slave = a;
	i.count = cnt;
	i.last = 0;
	i.buf = buf;
	if ((ioctl(ifd,I2CWRITE,&i)) < 0) {
		if (i2c_verbose)
			perror("I2CWRITE");
		i2c_error = I2C_ERR;
		goto error;
	}

	i.slave = a;
	i.count = 0;
	i.last = 0;
	i.buf = NULL;
/*
	if ((ioctl(ifd,I2CSTOP,&i)) < 0) {
*/
	if (ioctl(ifd,I2CSTOP) < 0) {
		if (i2c_verbose)
			perror("I2CSTOP");
		i2c_error = I2C_ERR;
		return(1);
	}
	return(0);

error:
	i.slave = a;
	i.count = 0;
	i.last = 0;
	i.buf = NULL;
	ioctl(ifd,I2CSTOP);
	return(1);
	
}

int
i2c_read(int ifd, unsigned char a, unsigned char *buf, int cnt)
{
	struct iiccmd i;

	if(i2c_verbose) {
		printf("i2c_read: ifd=%d cnt=%d a=%02x  ",ifd,cnt,a);
	}

	i.slave = a;
	i.count = 0;
	i.last = 0;
	i.buf = NULL;
	if ((ioctl(ifd,I2CSTART,&i)) < 0) {
		if (i2c_verbose)
			perror("I2CSTART");
		i2c_error = I2C_ERR;
		return(1);
	}

	i.slave = a;
	i.count = cnt;
	i.last = cnt;
	i.buf = buf;
	if ((ioctl(ifd,I2CREAD,&i)) < 0) {
		if (i2c_verbose)
			perror("I2CREAD");
		i2c_error = I2C_ERR;
		return(1);
	}

	i.slave = a;
	i.count = 0;
	i.last = 0;
	i.buf = NULL;
/*
	if ((ioctl(ifd,I2CSTOP,&i)) < 0) {
*/
	if (ioctl(ifd,I2CSTOP) < 0) {
		if (i2c_verbose)
			perror("I2CSTOP");
		i2c_error = I2C_ERR;
		return(1);
	}
	if (i2c_verbose) {
		printf("data=");
		printbuf(buf,cnt);
		printf("\n");
	}
	return(0);
}

void
i2c_perror(char *p)
{
	char *sep = "";

	if (p != NULL && *p != '\0')
		sep = ": ";
	if (i2c_error < 0 || i2c_error >= NI2CERROR) {
		fprintf(stderr,"%s%serror %d\n", p, sep, i2c_error);
	} else {
		fprintf(stderr,"%s%s%s\n", p, sep, i2c_error_tab[i2c_error]);
	}
}

void
printbuf(unsigned char *buf, int cnt)
{
	while (cnt--)
		printf("%02x ",*buf++);
}

