/*
 * VHF/UHF tuner test program
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <dev/iicbus/iic.h>

#include "global.h"
#include "cmdparse.h"
#include "i2c.h"

#ifdef	DOS
#define IICDEVICE	"2"		/* I2C device file              */
#endif
#ifdef	FreeBSD
#define IICDEVICE	"/dev/iic0"	/* I2C device file              */
#endif
#define	TCMK0601STARTUP	"./tcmk0601.cmd"

static char *Prompt = "tcmk0601> ";
int	ifd = -1;	/* I2C device	*/

int donop __ARGS((int argc,char *argv[],void *p));
int doexit __ARGS((int argc,char *argv[],void *p));
int dohelp __ARGS((int argc,char *argv[],void *p));
int dodelay __ARGS((int argc,char *argv[],void *p));
int doprint __ARGS((int argc,char *argv[],void *p));
int doset __ARGS((int argc,char *argv[],void *p));
int dodisplay __ARGS((int argc,char *argv[],void *p));
int docalc1243 __ARGS((int argc,char *argv[],void *p));
int dold1243 __ARGS((int argc,char *argv[],void *p));
int docalc3235 __ARGS((int argc,char *argv[],void *p));
int dold3235 __ARGS((int argc,char *argv[],void *p));
int dostat3235 __ARGS((int argc,char *argv[],void *p));
int doscan3235 __ARGS((int argc,char *argv[],void *p));
int dorxscan __ARGS((int argc,char *argv[],void *p));
int dorxset __ARGS((int argc,char *argv[],void *p));
int doreg3417 __ARGS((int argc,char *argv[],void *p));
int doreset3417 __ARGS((int argc,char *argv[],void *p));
int dostat3417 __ARGS((int argc,char *argv[],void *p));
int doinitdem3417 __ARGS((int argc,char *argv[],void *p));
int doinitdsp3417 __ARGS((int argc,char *argv[],void *p));
int doqpd __ARGS((int argc,char *argv[],void *p));
int doup __ARGS((int argc,char *argv[],void *p));

/* Command lookup and branch tables */
struct cmds Cmds[] = {
	"",		donop, 0,	NULLCHAR,		NULLCHAR,
	"delay",	dodelay, 0,	"Delay millisec",	NULLCHAR,
	"display",	dodisplay, 0,	"Display variables",	NULLCHAR,
	"exit",		doexit,	0,	"Exit program",		NULLCHAR,
	"help",		dohelp,	0,	"Print help",		NULLCHAR,
	"ld1243",	dold1243, 0,	"Load TA1243 data",	NULLCHAR,
	"ld3235",	dold3235, 0,	"Load CXA3235 data",	NULLCHAR,
	"calc1243",	docalc1243, 0,	"Calculate TA1243 N10-N1", NULLCHAR,
	"calc3235",	docalc3235, 0,	"Calculate CXA3235 N10-N1", NULLCHAR,
	"initdem3417",	doinitdem3417, 0,"Init MSP3417 demodulator", NULLCHAR,
	"initdsp3417",	doinitdsp3417, 0,"Init MSP3417 dsp",	NULLCHAR,
	"print",	doprint, 0,	"Print arguments",	NULLCHAR,
	"qpd",		doqpd,	0,	"Read Quasi-Peak detector", NULLCHAR,
	"reg3417",	doreg3417, 0,	"Read/Write MSP3417 register", NULLCHAR,
	"reset3417",	doreset3417, 0, "Reset MSP3417",	NULLCHAR,
	"rxscan",	dorxscan, 0,	"Rxscan from to step",	NULLCHAR,
	"rxset",	dorxset, 0,	"Set RX frequency",	NULLCHAR,
	"set",		doset, 0,	"Set/display variable",	NULLCHAR,
	"st3235",	dostat3235, 0,	"Calculate CXA3235 N10-N1", NULLCHAR,
	"st3417",	dostat3417, 0,	"Show MSP3417 status", NULLCHAR,
	"up",		doup, 0,	"Frz up 1 step",	NULLCHAR,
	"?",		dohelp,	0,	"Print help",		NULLCHAR,
	NULLCHAR,	NULLFP, 0,	NULLCHAR,
				"Unknown command; type \"?\" for list"
};

main(argc,argv)
int argc;
char *argv[];
{
	char	inbuf[256];
	FILE	*cmdf;

	/* Initialize */
#if	1
	if ((ifd = i2c_init(IICDEVICE)) < 0) {
		perror("");
		i2c_perror("init");
		exit(1);
	}
	ioctl(ifd, I2CSTOP);
#else
	printf("I2C bus NOT initialized\n");
#endif
	
	/* Read commands from startup command file */
	if ((cmdf = fopen(TCMK0601STARTUP,"r")) == NULL ) {
		fprintf(stderr,"No startup file\n");
	} else {
		while(fgets(inbuf,256,cmdf) != NULL) {
			rip(inbuf);
			if (strlen(inbuf) > 0)
				cmdparse(Cmds,inbuf,NULL);
		}
		fclose(cmdf);
	}

	/* Read commands from keyboard */
	for(;;) {
		printf(Prompt);
		if(fgets(inbuf,256,stdin) == NULL) {
			printf("\n");
			break;
		}
		rip(inbuf);
		if (strlen(inbuf) > 0)
			cmdparse(Cmds,inbuf,NULL);
	}
	return(0);
}

int
donop(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	return(0);
}
  
int
doexit(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	i2c_uninit(ifd);
	exit(0);
	return(0);	/* to keep bcc happy !! */
}
  
int
dohelp(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	register struct cmds *cmdp;
	int i;
	char buf[256];
	char **pargv;
	FILE *fp;
  
	puts("Main commands:\n");
        for(cmdp = Cmds;cmdp->name != NULL;cmdp++){
		printf("%-12s: %s\n",cmdp->name,cmdp->helpmsg);
        }
	return 0;
}

int
dodelay(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	unsigned long dly;

	if (argc != 2) {
		printf("Usage: delay millisec\n");
		return(1);
	}
	dly = atol(argv[1]);
	usleep(dly*1000);
}

int
doprint(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	int i;

	for(i=1 ; i < argc ; i++) {
		printf(argv[i]);
		if (i < argc-1)
			printf(" ");
	}
	printf("\n");
}

