/* MSP3417D commands
 */

#include <stdio.h>
#ifdef	MSDOS
#include <io.h>
#include <dos.h>
#endif
#include <fcntl.h>
#include <sys/stat.h>
#include <string.h>
#include <dev/iicbus/iic.h>

#include "global.h"
#include "i2c.h"
#include "msp3417.h"

extern	int	i2c_verbose;
extern	int	ifd;

/* Global variables */
 
/* Routines */

int
read_msp3417_control_register(unsigned char addr, unsigned short *status)
{
	struct iiccmd cmd;
	unsigned char data[16];

	cmd.slave = addr;
	cmd.count = 0;
	cmd.last = 0;
	cmd.buf = NULL;
	if (ioctl(ifd, I2CSTART, &cmd) != 0) {
		printf("I2CSTART_1 failed\n");
		perror("");
		return 1;
	}

	data[0] = 0x00;		/* CONTROL subaddress	*/
	cmd.slave = addr;
	cmd.count = 1;
	cmd.last = 0;
	cmd.buf = data;
	if(ioctl(ifd, I2CWRITE, &cmd) != 0) {
		printf("I2CWRITE failed\n");
		perror("");
		return 1;
	}

	cmd.slave = addr+1;
	cmd.count = 0;
	cmd.last = 0;
	cmd.buf = NULL;

	if (ioctl(ifd, I2CSTART, &cmd) != 0) {
		printf("I2CSTART_2 failed\n");
		perror("");
		return 1;
	}

	cmd.slave = addr+1;
	cmd.count = 2;
	cmd.last = 2;
	cmd.buf = data;

	if(ioctl(ifd, I2CREAD, &cmd) != 0) {
		printf("I2CREAD failed\n");
		perror("");
		return 1;
	}

	if(ioctl(ifd, I2CSTOP) != 0) {
		printf("I2CSTOP failed\n");
		perror("");
		return 1;
	}

	*status = ((data[0] << 8) & 0xffff) | (data[1] & 0xff);

	return 0;
}


int
read_msp3417_register(unsigned char addr, unsigned char sub_addr, 
			unsigned short reg, unsigned short *value)
{
	struct iiccmd cmd;
	unsigned char data[16];
	int r;

	cmd.slave = addr;
	cmd.count = 0;
	cmd.last = 0;
	cmd.buf = NULL;

	if (ioctl(ifd, I2CSTART, &cmd) != 0) {
		printf("I2CSTART_1 failed\n");
		perror("");
		return 1;
	}

	data[0] = sub_addr;		/* subaddress	*/
	data[1] = (reg >> 8) & 0xff;
	data[2] = reg & 0xff;
	cmd.slave = addr;
	cmd.count = 3;
	cmd.last = 0;
	cmd.buf = data;
	if((r = ioctl(ifd, I2CWRITE, &cmd)) != 0) {
		printf("I2CWRITE failed, r=%d\n",r);
		perror("");
		return 1;
	}

	cmd.slave = addr+1;
	cmd.count = 0;
	cmd.last = 0;
	cmd.buf = NULL;
	if (ioctl(ifd, I2CSTART, &cmd) != 0) {
		printf("I2CSTART_2 failed\n");
		perror("");
		return 1;
	}

	cmd.slave = addr+1;
	cmd.count = 2;
	cmd.last = 2;
	cmd.buf = data;
	if(ioctl(ifd, I2CREAD, &cmd) != 0) {
		printf("I2CREAD failed\n");
		perror("");
		return 1;
	}

	if(ioctl(ifd, I2CSTOP) != 0) {
		printf("I2CSTOP failed\n");
		perror("");
		return 1;
	}

	*value = ((data[0] << 8) & 0xffff) | (data[1] & 0xff);

	return 0;
}

int
write_msp3417_register(unsigned char addr, unsigned char sub_addr, 
			unsigned short reg, unsigned short value)
{
	struct iiccmd cmd;
	unsigned char data[16];
	int r;

	cmd.slave = addr;
	cmd.count = 0;
	cmd.last = 0;
	cmd.buf = NULL;

	if (ioctl(ifd, I2CSTART, &cmd) != 0) {
		printf("I2CSTART_1 failed\n");
		perror("");
		return 1;
	}

	data[0] = sub_addr;		/* subaddress	*/
	data[1] = (reg >> 8) & 0xff;	/* register */
	data[2] = reg & 0xff;
	data[3] = (value >> 8) & 0xff;	/* new value */
	data[4] = value & 0xff;
	cmd.slave = addr;
	cmd.count = 5;
	cmd.last = 0;
	cmd.buf = data;
	if((r = ioctl(ifd, I2CWRITE, &cmd)) != 0) {
		printf("I2CWRITE failed, r=%d\n",r);
		perror("");
		return 1;
	}

	if(ioctl(ifd, I2CSTOP) != 0) {
		printf("I2CSTOP failed\n");
		perror("");
		return 1;
	}

	return 0;
}

/* Commands */

int
doreg3417(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	unsigned short reg,value;
	unsigned char subaddr,data[16];

	if (argc < 3 || argc > 4) {
		printf("Usage: reg3417 \"dem\"|\"dsp\" regnum [ value ]\n");
		return 1;
	}
	if (strcmp(argv[1],"dem") == 0) {
		subaddr = WR_DEM;
	} else if (strcmp(argv[1],"dsp") == 0) {
		subaddr = WR_DSP;
	} else {
		printf("Usage: reg3417 \"dem\"|\"dsp\" regnum [ value ]\n");
		return 1;
	}
	reg = htoi(argv[2]);

	if (argc == 3) {
		/* read register */
		(void)read_msp3417_register(
				ADDR_MSP3417, subaddr+1, reg, &value);
		printf("%s 0x%04x : 0x%04x\n",
			subaddr==WR_DEM?"dem":"dsp",
			reg,
			value);
	} else {
		/* write register */
		value = htoi(argv[3]);
		(void)write_msp3417_register(
			ADDR_MSP3417,subaddr,reg,value);
		printf("%s 0x%04x <-- 0x%04x\n",
			subaddr==WR_DEM?"dem":"dsp",
			reg,
			value);
	}
}

int
doreset3417(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	unsigned char data[16];

	data[0] = 0x00;		/* Control Sub Address	*/
	data[1] = 0x80;		/* Reset bit 1		*/
	data[2] = 0x00;		/* 16bit regeister	*/

	if (i2c_write(ifd,ADDR_MSP3417,data,3)) {
		i2c_perror("write");
		return(1);
	}

	data[1] = 0x00;		/* Reset bit 0		*/

	if (i2c_write(ifd,ADDR_MSP3417,data,3)) {
		i2c_perror("write");
		return(1);
	}

	return(0);
}

struct dem_value_s {
	short	dem_reg;
	short	dem_val;
	char	*dem_desc;
};

struct dem_value_s dem_value[] = {
	{ 0x0020, 0x0001, "" },
};

#define NDEMREGS (sizeof(dem_value) / sizeof(struct dem_value_s))

struct dsp_value_s {
	short	dsp_reg;
	short	dsp_val;
	char	*dsp_desc;
};

struct dsp_value_s dsp_value[] = {
	{ 0x0000, 0x0000, "" },
	{ 0x0001, 0x0000, "" },
	{ 0x0003, 0x0000, "" },
	{ 0x0003, 0x0000, "" },
	{ 0x0004, 0x0000, "" },
	{ 0x0005, 0x0000, "" },
	{ 0x0006, 0x0000, "" },
	{ 0x0007, 0x6301, "" },
	{ 0x0008, 0x0000, "" },
	{ 0x0009, 0x0000, "" },
	{ 0x000a, 0x0020, "" },
	{ 0x000b, 0x0000, "" },
	{ 0x000c, 0x0030, "" },
	{ 0x000d, 0x0000, "" },
	{ 0x000e, 0x2403, "" },
	{ 0x000f, 0x0000, "" },
	{ 0x0010, 0x0000, "" },
	{ 0x0012, 0x0000, "" },
	{ 0x0013, 0x3f20, "" },
	{ 0x0015, 0x0000, "" },
	{ 0x0016, 0x0000, "" },
	{ 0x0017, 0x0000, "" },
	{ 0x0018, 0x0000, "" },
	{ 0x001b, 0x0000, "" },
	{ 0x001c, 0x0000, "" },
	{ 0x0020, 0x0000, "" },
	{ 0x0021, 0x0000, "" },
	{ 0x0022, 0x0000, "" },
	{ 0x0023, 0x0000, "" },
	{ 0x0024, 0x0000, "" },
	{ 0x0025, 0x0000, "" },
	{ 0x0029, 0x0000, "" },
	{ 0x002c, 0x0000, "" },
	{ 0x002d, 0x0000, "" },
	{ 0x0030, 0x0000, "" },
	{ 0x0031, 0x0000, "" },
	{ 0x0032, 0x0000, "" },
	{ 0x0033, 0x0000, "" },
	{ 0x0040, 0x0000, "" },
	{ 0x0041, 0x0000, "" },
};

#define NDSPREGS (sizeof(dsp_value) / sizeof(struct dsp_value_s))

int
dostat3417(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	unsigned short status,value;
	int r,i;
	int revision,prod,rom1,rom2;
	struct dsp_value_s *dspp;

	/* CONTROL register */
	if (read_msp3417_control_register(ADDR_MSP3417,&status) != 0) {
		printf("read_msp3417_control_register failed\n");
		return 1;
	}

	printf("CONTROL: 0x%04x\n",status);

	printf("\n");
	(void)read_msp3417_register(ADDR_MSP3417,RD_DEM,0x007e,&value);
	printf("STANDARD_RES: 0x%04x\n",value);
	(void)read_msp3417_register(ADDR_MSP3417,RD_DEM,0x0200,&value);
	printf("STATUS: 0x%04x\n",value);
/* Nor readable
	(void)read_msp3417_register(ADDR_MSP3417,RD_DEM,0x00bb,&value);
	printf("AD_CV: 0x%04x\n",value);
*/
	(void)read_msp3417_register(ADDR_MSP3417,RD_DEM,0x0083,&value);
	printf("MODE_REG: 0x%04x\n",value);

#if	0
	printf("\n");
	(void)read_msp3417_register(ADDR_MSP3417,RD_DSP,0x0000,&value);
	printf("Volume loudspeaker channel: 0x%04x\n",value);
	(void)read_msp3417_register(ADDR_MSP3417,RD_DSP,0x0007,&value);
	printf("Volume SCART1 output channel: 0x%04x\n",value);
	(void)read_msp3417_register(ADDR_MSP3417,RD_DSP,0x0008,&value);
	printf("Loudspeaker source select: 0x%04x\n",value);
	(void)read_msp3417_register(ADDR_MSP3417,RD_DSP,0x000a,&value);
	printf("SCART1 source select: 0x%04x\n",value);
	(void)read_msp3417_register(ADDR_MSP3417,RD_DSP,0x000c,&value);
	printf("Quasi-peak detector source select: 0x%04x\n",value);
	(void)read_msp3417_register(ADDR_MSP3417,RD_DSP,0x000d,&value);
	printf("Prescale SCART input: 0x%04x\n",value);
	(void)read_msp3417_register(ADDR_MSP3417,RD_DSP,0x000e,&value);
	printf("Prescale FM/AM input: 0x%04x\n",value);
	(void)read_msp3417_register(ADDR_MSP3417,RD_DSP,0x0010,&value);
	printf("Prescale NICAM: 0x%04x\n",value);
	(void)read_msp3417_register(ADDR_MSP3417,RD_DSP,0x0013,&value);
	printf("ACB: 0x%04x\n",value);
	(void)read_msp3417_register(ADDR_MSP3417,RD_DSP,0x0014,&value);
	printf("Beeper: 0x%04x\n",value);
	(void)read_msp3417_register(ADDR_MSP3417,RD_DSP,0x0029,&value);
	printf("Automatic Volume Correction: 0x%04x\n",value);
	
	printf("\n");
	(void)read_msp3417_register(ADDR_MSP3417,RD_DSP,0x0018,&value);
	printf("Stereo detection: 0x%04x\n",value);
	(void)read_msp3417_register(ADDR_MSP3417,RD_DSP,0x0019,&value);
	printf("Quasi-peak readout left: 0x%04x\n",value);
	(void)read_msp3417_register(ADDR_MSP3417,RD_DSP,0x001a,&value);
	printf("Quasi-peak readout right: 0x%04x\n",value);
	(void)read_msp3417_register(ADDR_MSP3417,RD_DSP,0x001e,&value);
	printf("MSP hardware code: 0x%04x\n",value);
	revision = value & 0xf;
	(void)read_msp3417_register(ADDR_MSP3417,RD_DSP,0x001f,&value);
	printf("MSP product code: 0x%04x : ",value);
	prod = (value >> 8) & 0xff;
	rom1 = (value >> 4) & 0xf;
	rom2 = value & 0xf;
	printf("MSP34%02d%c %c%c\n",prod,'A'+revision-1,'A'+rom1-1,'0'+rom2);
#endif
	dspp = dsp_value;
	for (i = 0 ; i < NDSPREGS ; i++) {
		(void)read_msp3417_register(ADDR_MSP3417,RD_DSP,
			dspp->dsp_reg,&value);
		printf("reg=0x%04x value=0x%04x\n",dspp->dsp_reg,value);
		dspp++;
	}
	return(0);
}

/* Init demodulator taken from msp3400.c from Gerd Knorr */
int
doinitdem3417(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	struct dem_value_s *demp;
	int i;

#if	0
	/* Start automatic standard selection */
	(void)write_msp3417_register(ADDR_MSP3417,WR_DEM,0x0020,0x0001);
#endif

	demp = dem_value;
	for (i = 0 ; i < NDEMREGS ; i++) {
		(void)write_msp3417_register(ADDR_MSP3417,WR_DEM,
			demp->dem_reg,demp->dem_val);
		demp++;
	}
}

/* Init dsp */
int
doinitdsp3417(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	struct dsp_value_s *dspp;
	int i;

	dspp = dsp_value;
	for (i = 0 ; i < NDSPREGS ; i++) {
		(void)write_msp3417_register(ADDR_MSP3417,RD_DSP,
			dspp->dsp_reg,dspp->dsp_val);
		dspp++;
	}
}

/* Read Quasi-Peak Detector registers */
int
doqpd(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	short qsp_l, qsp_r;

	(void)read_msp3417_register(ADDR_MSP3417,RD_DSP,0x0019,&qsp_l);
	(void)read_msp3417_register(ADDR_MSP3417,RD_DSP,0x001a,&qsp_r);
	printf("Quasi-Peak Detector: left=%d right=%d\n",qsp_l,qsp_r);
}

