/*  Variable set/display routines
 *
 *  (c) Copyright 1996,1997 Jakob Schripsema , PA3HDF  All Rights Reserved
 *  Permission for non-commercial use is hereby granted provided this notice
 *  is retained.
 */


#include <stdio.h>
#include <string.h>
#include "vars.h"

/*
 * Global variables
 */
long	if_freq = 39800;		/* IF frequency in KHz	*/
long	fstep = 50;			/* RX frequency step	*/
long	frx = 0;			/* RX frequency		*/
long	ftx = 0;			/* TX frequency		*/

#define	T_NONE		0
#define	T_INT		1
#define	T_LONG		2
#define	T_ON_OFF	3
#define	T_STRING	4

struct var_desc {
	char *v_name;	/* Name of variable */
	char *v_mem;	/* Memory location	*/
	int v_type;	/* Variable type	*/
};

/* Variable table. The order of this table controls the
 * output order of the 'display' command
 */

struct	var_desc var_tab[] = {
	{	"frx",		(char *)(&frx),			T_LONG	},
	{	"ftx",		(char *)(&ftx),			T_LONG	},
	{	"if",		(char *)(&if_freq),		T_LONG	},
	{	"step",		(char *)(&fstep),		T_LONG	},
	{	(char *)0,	NULL,				T_NONE	},
};

/* Set/display variable */
int
doset (int argc,char *argv[],void *p)
{
	register struct var_desc *vp;
	int found = 0;
	int i,len;
  
	/* Strip off first token and pass rest of line to subcommand */
        if (argc < 1) {
		puts("SET - Don't know what to do?\n");
		return -1;
	}

	if (strcmp(argv[1],"?") == 0) {
		for(vp = var_tab;vp->v_name != NULL;vp++){
			printf("%s\n",vp->v_name);
		}
		return 0;
	}
	for(vp = var_tab;vp->v_name != NULL;vp++){
		if(strncmp(argv[1],vp->v_name,strlen(argv[1])) == 0){
			found = 1;
			break;
		}
	}
	if(!found){
		printf("unknown variable %s\n",argv[1]);
		return -1;
        }

	if (argc < 3) {
		/* Display variable */
		switch (vp->v_type) {
		case T_INT:
			printf("%s %d\n",vp->v_name,*(int *)vp->v_mem);
			break;
		case T_LONG:
			printf("%s %ld\n",vp->v_name,*(long *)vp->v_mem);
			break;
		case T_ON_OFF:
			printf("%s %s\n",vp->v_name,
				*(int *)vp->v_mem ? "on" : "off");
			break;
		case T_STRING:
			printf("%-13s : %s\n",
					vp->v_name,*(char **)vp->v_mem);
			break;
		default:
			printf("%s: unknown type %d\n",vp->v_name,vp->v_type);
		}
	} else {
		/* Set variable */
		switch (vp->v_type) {
		case T_INT:
			sscanf(argv[2],"%d",(int *)vp->v_mem);
			break;
		case T_LONG:
			sscanf(argv[2],"%ld",(long *)vp->v_mem);
			break;
		case T_ON_OFF:
			/* Hack (more or less ..) */
			if (strcmp(argv[2],"on") == 0)
				*(int *)vp->v_mem = 1;
			else
				*(int *)vp->v_mem = 0;
			break;
		case T_STRING:
			*(char **)vp->v_mem = strdup(argv[2]);
			break;
		default:
			printf("%s: unknown type %d\n",vp->v_name,vp->v_type);
		}

		/* Notify driver of possible new parameters */
		/* device_update(); */
	}

	return 0;
}

int
dodisplay (int argc,char *argv[],void *p)
{
	struct var_desc *vp;
	int cnt = 0;

	/* Display non-string variables */
	vp = var_tab;
	while (vp->v_name) {
		switch (vp->v_type) {
		case T_INT:
			printf("%-13s : %-6d    ",vp->v_name,*(int *)vp->v_mem);
			break;
		case T_LONG:
			printf("%-13s : %-6ld    ",
					vp->v_name,*(long *)vp->v_mem);
			break;
		case T_ON_OFF:
			printf("%-13s : %-6s    ",vp->v_name,
				*(int *)vp->v_mem ? "on" : "off");
			break;
		case T_STRING:
			vp++;
			continue;
		default:
			printf("%s: unknown type %d\n",vp->v_name,vp->v_type);
		}
		if (++cnt == 3) {
			printf("\n");
			cnt = 0;
		}
		vp++;
	}
	if (cnt != 0)
		printf("\n");

	/* Display string variables */
	vp = var_tab;
	while (vp->v_name) {
		switch (vp->v_type) {
		case T_STRING:
			printf("%-13s : %s\n",
					vp->v_name,*(char **)vp->v_mem);
			break;
		default:
			vp++;
			continue;
		}
		vp++;
	}
	return 0;
}

