#!/usr/bin/env python

import wx
#import wx.lib.plot as plot
import os
import time
#import Numeric as _Numeric
import math

import i2c_tiny_usb
import ta1243
import cxa3235
import msp3417

ID_ABOUT=101
ID_OPEN=102
ID_BUTTON1=110
ID_EXIT=200

KEY_FUNC = 100
KEY_SCAN = 101

OFFSET_X = 10.0
OFFSET_Y = 10.0
FMIN = 50.0
FMAX = 850.0

class SpectrumDisplay(wx.Window):
    def __init__(self, parent, id=wx.ID_ANY, pos=wx.DefaultPosition,
			size=wx.DefaultSize, style=0, name="SpectrumDisplay"):
	#wx.Window.__init__(self, parent, id, pos, size, style, name)
	super(SpectrumDisplay,self).__init__(parent,
		size=(820,120), style=wx.NO_FULL_REPAINT_ON_RESIZE)
	self.parent = parent

	# Create some mouse events for zooming
	self.Bind(wx.EVT_LEFT_DOWN, self.OnMouseLeftDown)
	#self.canvas.Bind(wx.EVT_LEFT_UP, self.OnMouseLeftUp)
	#self.canvas.Bind(wx.EVT_MOTION, self.OnMotion)
	#self.canvas.Bind(wx.EVT_LEFT_DCLICK, self.OnMouseDoubleClick)
	#self.canvas.Bind(wx.EVT_RIGHT_DOWN, self.OnMouseRightDown)

	# set curser as cross-hairs
	self.SetCursor(wx.CROSS_CURSOR)

	self.Bind(wx.EVT_PAINT, self.OnPaint)
	#self.Bind(wx.EVT_SIZE, self.OnSize)

	(self.width, self.height)  = self.GetClientSize()
	#print "Size", self.width, self.height
	self.buffer = wx.EmptyBitmap(self.width, self.height)
	#self._setSize()
	dc = wx.BufferedDC(wx.ClientDC(self), self.buffer)
	dc.SetBackground(wx.Brush('white'))
	dc.Clear()

    def OnPaint(self, event):
	# All that is needed here is to draw the buffer to screen
	#print "OnPaint"
	dc = wx.BufferedPaintDC(self, self.buffer)
                
    def OnSize(self,event):
	#print "OnSize"
	pass

    def OnMouseLeftDown(self, event):
	x, y = event.GetPosition()
	x = float(x - OFFSET_X)
	f = self.fmin + (self.fmax - self.fmin)*(x / 800.0)
	s = "F = %.3f MHz" % (f / 1000.0)
	self.parent.SetStatusText(s)
	#print "Frequency:", f

    def Clear(self):
	dc = wx.BufferedDC(wx.ClientDC(self), self.buffer)
	dc.Clear()

    def SetRange(self, min, max):
	self.fmin = float(min)
	self.fmax = float(max)

    def DrawAxis(self):
	#print "DrawAxis", self.width, self.height
	dc = wx.BufferedDC(wx.ClientDC(self), self.buffer)
	dc.Clear()
	dc.DrawLine(OFFSET_X, OFFSET_Y,
				self.width - OFFSET_X, OFFSET_Y)
	dc.DrawLine(self.width - 10.0, 10.0, self.width - 10.0, self.height - 10.0)
	dc.DrawLine(self.width - 10.0, self.height - 10.0, 10.0, self.height - 10.0)
	dc.DrawLine(10.0, self.height - 10.0, 10.0, 10.0)

    def Plot1(self, f, s):
	dc = wx.BufferedDC(wx.ClientDC(self), self.buffer)
	if (f < self.fmin) or (f > self.fmax):
	    print "Skipped"
	    return
	x = (float(f) - self.fmin) / (self.fmax - self.fmin) * 800.0
	# Linear scale
	#print "Plot1",f,s, x
	#dc.DrawLine(x+OFFSET_X, self.height - OFFSET_Y,
	#			x+OFFSET_X, self.height - s - OFFSET_Y)
	# Log scale
	if s < 1:
	    s = 1
	logs = math.log10(s)
	if logs < 0:
	    logs = 0
	#print "Plot1",f,s, logs
	logs *= 40.0
	dc.DrawLine(x+OFFSET_X, self.height - OFFSET_Y,
				x+OFFSET_X, self.height - logs - OFFSET_Y)

class FrequencySliders(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent, -1)
 
        box = wx.StaticBox(self, -1, "Limits")
        bsizer = wx.StaticBoxSizer(box, wx.VERTICAL)

	# Tuner range 50-850 MHz
	# Minimum span 40 MHz
        self.low = wx.Slider(self,-1, value=50, minValue=50, maxValue=850,style=wx.SL_HORIZONTAL|wx.SL_LABELS, size=(600,40))
        self.low.Bind(wx.EVT_SLIDER, self.LowLimitSet)
        bsizer.Add(self.low, 0, wx.TOP|wx.CENTER)
        self.high = wx.Slider(self,-1, value=850, minValue=50, maxValue=850,style=wx.SL_HORIZONTAL|wx.SL_LABELS, size=(600,40))
        bsizer.Add(self.high, 0, wx.TOP|wx.CENTER)
        self.high.Bind(wx.EVT_SLIDER, self.HighLimitSet)

        border = wx.BoxSizer()
        border.Add(bsizer, 1, wx.EXPAND|wx.ALL, 1)
        self.SetSizer(border)

    def GetLimits(self):
	low = self.low.GetValue()
	high = self.high.GetValue()
	return (low, high)

    def LowLimitSet(self,e):
	low = self.low.GetValue()
	if low > 810:
	    self.low.SetValue(810)
	high = self.high.GetValue()
	if high < low + 40:
	    self.high.SetValue(low + 40)

    def HighLimitSet(self,e):
	high = self.high.GetValue()
	if high < 90:
	    self.high.SetValue(90)
	low = self.low.GetValue()
	if low > high - 40:
	    self.low.SetValue(high - 40)

class BandScopeWindow(wx.Frame):
    def __init__(self,parent,id,title):
        wx.Frame.__init__(self,parent,wx.ID_ANY, title)
	# Need this for Windows
	self.SetBackgroundColour((220, 218, 213, 255))

	# GUI components
        #self.control = wx.TextCtrl(self, 1, style=wx.TE_MULTILINE)
        self.CreateStatusBar() # A Statusbar in the bottom of the window

        # Menu bar
        filemenu= wx.Menu()
        filemenu.Append(ID_EXIT,"E&xit"," Terminate the program")
        # Creating the menubar.
        menuBar = wx.MenuBar()
        menuBar.Append(filemenu,"&File") # Adding the "filemenu" to the MenuBar
        self.SetMenuBar(menuBar)  # Adding the MenuBar to the Frame content.
        wx.EVT_MENU(self, ID_EXIT, self.OnExit)

	# A status bar to tell people what's happening
	self.CreateStatusBar(1)

	# Spectrum Display
	self.spectrumdisplay = SpectrumDisplay(self)
	#self.spectrumdisplay.Show()

	# Low, high limit
	self.freqLimits = FrequencySliders(self)

	# Buttons
	self.scanButton = wx.Button(self, KEY_SCAN, "Scan")
	self.scanButton.Bind(wx.EVT_BUTTON, self.OnButtonScan)

	# Main layout
	self.sizer = wx.GridBagSizer(0,0)
	self.sizer.Add(self.spectrumdisplay, (0,0), (1,6), wx.EXPAND)
	#self.sizer.Add(self.spectrumdisplay, (0,0), (1,5), 0)
	self.sizer.Add(self.freqLimits, (1,0), (1,5), wx.EXPAND)
	self.sizer.Add(self.scanButton, (1,5), (1,1), wx.EXPAND)
	#self.sizer.Add(self.channels, (3,3), (1,4), wx.EXPAND)
	#self.sizer.Add(self.volume, (4,3), (1,4), wx.EXPAND)
        #Layout sizers
        self.SetSizer(self.sizer)
        self.SetAutoLayout(1)
        self.sizer.Fit(self)
        self.Show(1)

	self.spectrumdisplay.DrawAxis()
	#for f in range(50,850):
	#    s = (f - 50) % 50
	#    self.spectrumdisplay.Plot1(f,s)

    def OnExit(self,e):
        self.Close(True)  # Close the frame.

    def OnModeSelect(self,e):
	m = self.currentmode.GetSelection()
	if m == MODE_FM:
	    msp3417.WriteDemRegister(0x0083, 0x0000)
	    print "Mode FM"
	elif m == MODE_AM:
	    msp3417.WriteDemRegister(0x0083, 0x816e)
	    print "Mode AM"

    def OnButtonScan(self, e):
	#print "OnButtonScan"
	fmin,fmax = self.freqLimits.GetLimits()
	fmin *= 1000
	fmax *= 1000
	fstep = (fmax - fmin) / 800
	# Round down
	fstep = int(fstep / 50) * 50
	#print "Scanning from %d to %d" % (fmin, fmax)
	s = "Scanning from %d MHz to %d MHz" % (fmin/1000, fmax/1000)
	self.SetStatusText(s)
	self.Scan(fmin, fmax, fstep)
	self.spectrumdisplay.Show()

    def Scan(self, fmin, fmax, fstep):
	f = fmin
	np = 0
	t1 = time.time()
	self.spectrumdisplay.SetRange(fmin, fmax)
	self.spectrumdisplay.DrawAxis()
	while f <= fmax:
	    cxa3235Dev.SetFrequency(f)
	    l = msp3417Dev.QpdLeft()
	    #print "f",f,"l",l
	    if l > 1000:
		l = 1000
	    self.spectrumdisplay.Plot1(f,l/10)
	    f += fstep
	    np += 1
	t2 = time.time()
	s = "Scan of %d points took %.3f sec" % (np, t2-t1)
	self.SetStatusText(s)

# Main

TA1243_ADDR = 0x65
CXA3235_ADDR = 0x60
MSP3417_ADDR = 0x40

i2cTinyUsbDev = i2c_tiny_usb.I2cTinyUsbDevice()
handle = i2cTinyUsbDev.open()
i2cTinyUsbDev.setDelay(10)

ta1243Dev = ta1243.Ta1243Device(i2cTinyUsbDev, TA1243_ADDR)
cxa3235Dev = cxa3235.Cxa3235Device(i2cTinyUsbDev, CXA3235_ADDR)
msp3417Dev = msp3417.Msp3417Device(i2cTinyUsbDev, MSP3417_ADDR)

msp3417Dev.SetVolume(0)
msp3417Dev.SetMode(msp3417.MODE_AM)

app = wx.PySimpleApp()
frame = BandScopeWindow(None, -1, "BandScope")
#frame.Scan(50000, 850000, 1000)
app.MainLoop()

