#
# i2c_tiny_usb module
#

import sys
import usb

class DeviceDescriptor(object) :
    def __init__(self, vendor_id, product_id, interface_id) :
        self.vendor_id = vendor_id
        self.product_id = product_id
	self.interface_id = interface_id

    def getDevice(self) :
        """
        Return the device corresponding to the device descriptor if it is
        available on a USB bus.  Otherwise, return None.  Note that the
        returned device has yet to be claimed or opened.
        """
        buses = usb.busses()
        for bus in buses :
            for device in bus.devices :
                if device.idVendor == self.vendor_id :
                    if device.idProduct == self.product_id :
                        return device
        return None

class I2cTinyUsbDevice(object) :
    
    I2CTINYUSB_VID = 0x0403
    I2CTINYUSB_PID = 0xc631
    I2CTINYUSB_IID = 0

    CMD_GET_FUNC = 1
    CMD_SET_DELAY = 2
    CMD_GET_STATUS = 3

    CMD_I2C_BEGIN = 1
    CMD_I2C_END = 2
    CMD_I2C_IO = 4

    I2C_M_RD = 1	# ???

    STATUS_IDLE = 0
    STATUS_ADDRESS_ACK = 1
    STATUS_ADDRESS_NAK = 2

    def __init__(self) :
        self.device_descriptor = DeviceDescriptor(I2cTinyUsbDevice.I2CTINYUSB_VID,
                                                  I2cTinyUsbDevice.I2CTINYUSB_PID,
                                                  I2cTinyUsbDevice.I2CTINYUSB_IID)
        self.device = self.device_descriptor.getDevice()
        self.handle = None

    def open(self) :
        #self.device = self.device_descriptor.getDevice()
        self.handle = self.device.open()
        if sys.platform == 'darwin' :
            # XXX : For some reason, Mac OS X doesn't set the
            # configuration automatically like Linux does.
            self.handle.setConfiguration(1)
        self.handle.claimInterface(self.device_descriptor.interface_id)
	return self.handle

    def close(self) :
        self.handle.releaseInterface()

    def getFunctionality(self) :
	reqType = usb.TYPE_CLASS | usb.ENDPOINT_IN
	reqBuffer = self.handle.controlMsg(reqType, self.CMD_GET_FUNC, 4)
	return (reqBuffer[3]<<24) + (reqBuffer[2]<<16) + (reqBuffer[1]<<8) + reqBuffer[0]

    def setDelay(self, delay) :
	reqType = usb.TYPE_VENDOR
	self.handle.controlMsg(reqType, self.CMD_SET_DELAY, 0, value = delay)

    def getStatus(self) :
	reqType = usb.TYPE_CLASS | usb.ENDPOINT_IN
	return self.handle.controlMsg(reqType, self.CMD_GET_STATUS, 1)

    def i2cProbe(self, address) :
	reqType = usb.TYPE_CLASS | usb.ENDPOINT_IN
	cmd = self.CMD_I2C_IO + self.CMD_I2C_BEGIN + self.CMD_I2C_END
	self.handle.controlMsg(reqType, cmd, 0, index=address)
	return self.getStatus()[0] == self.STATUS_ADDRESS_ACK
	
    def i2cRead(self,address, len) :
	# Write
	reqType = usb.TYPE_CLASS | usb.ENDPOINT_IN
	cmd = self.CMD_I2C_IO + self.CMD_I2C_BEGIN + self.CMD_I2C_END
	data = self.handle.controlMsg(reqType, cmd, len, value=self.I2C_M_RD, index=address)

	# Check status
	if self.getStatus()[0] == self.STATUS_ADDRESS_ACK:
	    return data
	else:
	    print"i2cRead(",hex(address),msg,") failed"
	    return -1

    def i2cWrite(self,address, msg) :
	# Write
	reqType = usb.TYPE_CLASS
	cmd = self.CMD_I2C_IO + self.CMD_I2C_BEGIN + self.CMD_I2C_END
	nBytes = self.handle.controlMsg(reqType, cmd, msg, index=address)

	# Check status
	if self.getStatus()[0] == self.STATUS_ADDRESS_ACK:
	    return nBytes
	else:
	    print"i2cWrite(",hex(address),msg,") failed"
	    return -1

    def i2cWriteRestartRead(self,address,msg,len) :
	reqType = usb.TYPE_CLASS
	cmd = self.CMD_I2C_IO + self.CMD_I2C_BEGIN
	nBytes = self.handle.controlMsg(reqType, cmd, msg, index=address)

	if self.getStatus()[0] != self.STATUS_ADDRESS_ACK:
	    print "i2cWriteRestartRead(",hex(address),msg,len,") failed"

	reqType = usb.TYPE_CLASS | usb.ENDPOINT_IN
	cmd = self.CMD_I2C_IO + self.CMD_I2C_END
	reqBuffer = self.handle.controlMsg(reqType, cmd, len, value=self.I2C_M_RD, index=address)
	return reqBuffer

