#!/usr/bin/env python
#
# MSP3417 IF decoder
#

import usb
import i2c_tiny_usb

MSP3417_ADDR = 0x40

MODE_FM=0
MODE_AM=1

class Msp3417Device(object) :

    def __init__(self,i2cDev,address) :
	self.i2cDev = i2cDev
	self.handle = i2cDev.handle
	self.address = address

	# Probe MSP3417
	if self.i2cDev.i2cProbe(self.address) == True:
	    temp = self.ReadDspRegister(0x001e)
	    hardwareVersion = temp >> 8
	    majorRevision = temp & 0xff
	    temp = self.ReadDspRegister(0x001f)
	    productCode = temp >> 8
	    romVersion = temp & 0xff
	    print "MSP34%02d%c-%c%c detected" % (productCode,
						ord('A')+majorRevision-1,
						ord('A')+hardwareVersion-1,
						chr(romVersion + 0x10))

	# Reset MSP34xx
	self.WriteControlRegister(0x8000)
	self.WriteControlRegister(0x0000)

	# Default Demodulator settings
	# AD_CV
	self.WriteDemRegister(0x00bb, 0x00d0)
	# FIR1
	self.WriteDemRegister(0x0001, 0x006b)
	self.WriteDemRegister(0x0001, 0x004e)
	self.WriteDemRegister(0x0001, 0x0006)
	self.WriteDemRegister(0x0001, 0x0004)
	self.WriteDemRegister(0x0001, 0x00f8)
	self.WriteDemRegister(0x0001, 0x00f8)
	# FIR2
	self.WriteDemRegister(0x0005, 0x0004)
	self.WriteDemRegister(0x0005, 0x0040)
	self.WriteDemRegister(0x0005, 0x0000)
	self.WriteDemRegister(0x0005, 0x006b)
	self.WriteDemRegister(0x0005, 0x004e)
	self.WriteDemRegister(0x0005, 0x0006)
	self.WriteDemRegister(0x0005, 0x0004)
	self.WriteDemRegister(0x0005, 0x00f8)
	self.WriteDemRegister(0x0005, 0x00f8)
	# MODE_REG
	self.WriteDemRegister(0x0083, 0x0000)
	# DCO1
	self.WriteDemRegister(0x0093, 0x038e)
	self.WriteDemRegister(0x009b, 0x04c6)
	# DCO2
	self.WriteDemRegister(0x00a3, 0x038e)
	self.WriteDemRegister(0x00ab, 0x04c6)
	# AUTO_FM/AM
	self.WriteDemRegister(0x0021, 0x0801)

	# Default DSP settings
	# DSP source
	self.WriteDspRegister(0x000a, 0x0020)
	# DSP matrix
	# H: Volume prescale FM
	self.WriteDspRegister(0x000e, 0x1800)
	# DSP SCART1 volume
	self.WriteDspRegister(0x0007, 0x6301)
	# DSP SCART switching
	self.WriteDspRegister(0x0013, 0x3f20)
	# QPD
	self.WriteDspRegister(0x000c, 0x0030)


    def ReadControlRegister(self):
	data = self.i2cDev.i2cWriteRestartRead(self.address, [0], 2)
	return (data[0] << 8) | data[1]

    def WriteControlRegister(self, data):
	wdata = [0x00, data >> 8, data & 0xff]
	self.i2cDev.i2cWrite(self.address, wdata)

    def ReadDemRegister(self):
	wdata = [ 0x11, reg >> 8, reg & 0xff]
	rdata = self.i2cDev.i2cWriteRestartRead(self.address, wdata, 2)
	return (rdata[0] << 8) | rdata[1]

    def WriteDemRegister(self, reg, data):
	wdata = [ 0x10, reg >> 8, reg & 0xff, data >> 8, data & 0xff]
	self.i2cDev.i2cWrite(self.address, wdata)

    def ReadDspRegister(self, reg):
	wdata = [ 0x13, reg >> 8, reg & 0xff]
	rdata = self.i2cDev.i2cWriteRestartRead(self.address, wdata, 2)
	return (rdata[0] << 8) | rdata[1]

    def WriteDspRegister(self, reg, data):
	wdata = [ 0x12, reg >> 8, reg & 0xff, data >> 8, data & 0xff]
	self.i2cDev.i2cWrite(self.address, wdata)

    def setFrequency(self,f) :
	# F in KHz
	n = int((f - 250) / 1000)
	i2cData = [ 0x8e,
		    0xc8,
		    0x40 | (n >> 4),
		    0x04 | ((n & 15) << 4)
		  ]
	self.i2cDev.i2cWrite(self.address, i2cData)
	if self.i2cDev.getStatus()[0] == self.i2cDev.STATUS_ADDRESS_ACK :
	    print "TA1243 loaded, f =", f, "KHz"
	else :
	    print "TA1243 not loaded"

    def SetVolume(self, v):
	self.WriteDspRegister(0x0007, (int(v*127/100) << 8) | 1)

    def SetMode(self, m):
	if m == MODE_FM:
	    self.WriteDemRegister(0x0083, 0x0000)
	elif m == MODE_AM:
	    self.WriteDemRegister(0x0083, 0x816e)

    def Qpd(self):
	l = self.ReadDspRegister(0x0019)
	r = self.ReadDspRegister(0x001a)
	return (l,r)

    def QpdLeft(self):
	l = self.ReadDspRegister(0x0019)
	return l

if __name__ == "__main__" :
    i2cTinyUsbDev = i2c_tiny_usb.I2cTinyUsbDevice()
    handle = i2cTinyUsbDev.open()
    i2cTinyUsbDev.setDelay(10)

    msp3417Dev = Msp3417Device(i2cTinyUsbDev, MSP3417_ADDR)
    # Beep 1KHz (check on LoudSpeaker outputs)
    msp3417Dev.WriteDspRegister(0x0014, 0x4040)
    print "DSP register 0x0014:", hex(msp3417Dev.ReadDspRegister(0x0014))

