#!/usr/bin/env python

import wx
import os

import i2c_tiny_usb
import ta1243
import cxa3235
import msp3417

ID_ABOUT=101
ID_OPEN=102
ID_BUTTON1=110
ID_EXIT=200

KEY_FUNC=100
KEY_DOT=0x2e
KEY_HASH=0x23
KEY_CMD0=0x80
KEY_CMD1=0x81
KEY_CMD2=0x82
KEY_FSET=0x83

#MODE_FM=0
#MODE_AM=1

presets = [
		["Radio1", "91.8", msp3417.MODE_FM],
		["Radio2", "88.0", msp3417.MODE_FM],
		["Radio3", "88.6", msp3417.MODE_FM],
		["Qmusic", "100.4", msp3417.MODE_FM],
		["SkyRadio", "101.0", msp3417.MODE_FM],
		["Veronica", "103.4", msp3417.MODE_FM],
		["Eelde Tower", "118.7", msp3417.MODE_AM],
		["Eelde Approach", "120.3", msp3417.MODE_AM],
		["Eelde ATIS", "133.55", msp3417.MODE_AM],
		["Eelde VOR", "112.4", msp3417.MODE_AM],
	]

class Presets(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent, -1)
        box = wx.StaticBox(self, -1, "Presets")
        bsizer = wx.StaticBoxSizer(box, wx.VERTICAL)

	self.presets = []
	#for p in presets:
	#    self.presets += [p[0]]
	self.presets = [ p[0] for p in presets ]
	self.presets = wx.ComboBox(self, -1, choices=self.presets, style=wx.CB_READONLY)
        bsizer.Add(self.presets, 0, wx.TOP|wx.CENTER)
	self.Bind(wx.EVT_COMBOBOX, parent.OnPresetSelect)

        border = wx.BoxSizer()
        border.Add(bsizer, 1, wx.EXPAND|wx.ALL, 5)
        self.SetSizer(border)

    def ClearPreset(self):
	print "ClearPreset"
	# HACK, don't know how to do this properly
	# Need to clear 'Preset' field
	self.presets.SetItems([""])
	self.presets.SetSelection(0)
	self.presets.SetItems([ p[0] for p in presets ])

    def GetPreset(self):
	return self.presets.GetCurrentSelection()

class KeyBoard(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent, -1)

	self.parent = parent
	self.kbBuffer = ""

	kbsizer  = wx.GridBagSizer(3, 4)
	self.key1 = wx.Button(self, 0x31, "1", size=(30,30))
        kbsizer.Add(self.key1, (0,1), (1,1))
	self.Bind(wx.EVT_BUTTON, self.OnKey)
	self.key2 = wx.Button(self, 0x32, "2", size=(30,30))
        kbsizer.Add(self.key2, (0,2), (1,1))
	self.key3 = wx.Button(self, 0x33, "3", size=(30,30))
        kbsizer.Add(self.key3, (0,3), (1,1))

	self.key4 = wx.Button(self, 0x34, "4", size=(30,30))
        kbsizer.Add(self.key4, (1,1), (1,1))
	self.key5 = wx.Button(self, 0x35, "5", size=(30,30))
        kbsizer.Add(self.key5, (1,2), (1,1))
	self.key6 = wx.Button(self, 0x36, "6", size=(30,30))
        kbsizer.Add(self.key6, (1,3), (1,1))

	self.key7 = wx.Button(self, 0x37, "7", size=(30,30))
        kbsizer.Add(self.key7, (2,1), (1,1))
	self.key8 = wx.Button(self, 0x38, "8", size=(30,30))
        kbsizer.Add(self.key8, (2,2), (1,1))
	self.key9 = wx.Button(self, 0x39, "9", size=(30,30))
        kbsizer.Add(self.key9, (2,3), (1,1))

	self.keys = wx.Button(self, KEY_DOT, ".", size=(30,30))
        kbsizer.Add(self.keys, (3,1), (1,1))
	self.key0 = wx.Button(self, 0x30, "0", size=(30,30))
        kbsizer.Add(self.key0, (3,2), (1,1))
	self.keyh = wx.Button(self, KEY_HASH, "#", size=(30,30))
        kbsizer.Add(self.keyh, (3,3), (1,1))

	self.keys = wx.Button(self, KEY_CMD0, "cmd0", size=(60,30))
        kbsizer.Add(self.keys, (0,4), (1,1))
	self.keys = wx.Button(self, KEY_CMD1, "cmd1", size=(60,30))
        kbsizer.Add(self.keys, (1,4), (1,1))
	self.keys = wx.Button(self, KEY_CMD2, "cmd2", size=(60,30))
        kbsizer.Add(self.keys, (2,4), (1,1))
	self.keys = wx.Button(self, KEY_FSET, "FSET", size=(60,30))
        kbsizer.Add(self.keys, (3,4), (1,1))

        self.SetSizer(kbsizer)

    def OnKey(self, e):
	id = e.GetId()
	if id == KEY_HASH:
	    self.parent.IgnoreKeyboard()
	    self.kbBuffer = ""
	elif id < KEY_FUNC:
	    self.kbBuffer = self.kbBuffer + chr(id)
	    self.parent.UpdateDisplay(self.kbBuffer)
	elif id == KEY_FSET:
	    self.parent.SetFrequency(self.kbBuffer)
	    self.kbBuffer = ""
	elif id == KEY_CMD0:
	    self.parent.OnCmd0()
	    self.kbBuffer = ""
	elif id == KEY_CMD1:
	    self.parent.OnCmd1()
	    self.kbBuffer = ""
	elif id == KEY_CMD2:
	    self.parent.OnCmd2()
	    self.kbBuffer = ""
	else:
	    print "Key",id

    def GetBuffer(self):
	return self.kbBuffer

class FrequencyDial(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent, -1)

        box = wx.StaticBox(self, -1, "Frequency")
        bsizer = wx.StaticBoxSizer(box, wx.VERTICAL)

	self.f = 888888888
        self.t = wx.StaticText(self, -1, self.Format(self.f), style=wx.ALIGN_LEFT)
        font = wx.Font(48, wx.NORMAL, wx.NORMAL, wx.NORMAL)
        self.t.SetFont(font)
        bsizer.Add(self.t, 0, wx.TOP|wx.LEFT, 10)

        border = wx.BoxSizer()
        border.Add(bsizer, 1, wx.EXPAND|wx.ALL, 5)
        self.SetSizer(border)

    def SetText(self, s):
	self.t.SetLabel(label=s)

    def Format(self,f):
	mhz = int(f/1000000)
	khz = int((f - mhz*1000000) / 1000)
	hz = f - mhz*1000000 - khz*1000
	return "% 3d.%03d.%1d" % (mhz, khz,hz/100)

    def GetFrequency(self):
	return self.f

    def SetFrequency(self, f):
	self.f = f
	self.t.SetLabel(label=self.Format(f))
	self.Show()

class VolumeSlider(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent, -1)

        box = wx.StaticBox(self, -1, "Volume")
        bsizer = wx.StaticBoxSizer(box, wx.VERTICAL)

	self.vol = wx.Slider(self,-1, value=77, minValue=0, maxValue=100,style=wx.SL_HORIZONTAL|wx.SL_LABELS, size=(200,40))
        bsizer.Add(self.vol, 0, wx.TOP|wx.CENTER)
	self.Bind(wx.EVT_SLIDER, parent.OnVolumeSet)

        border = wx.BoxSizer()
        border.Add(bsizer, 1, wx.EXPAND|wx.ALL, 5)
        self.SetSizer(border)

	msp3417Dev.WriteDspRegister(0x0007, 0x6301)

    def GetVolume(self):
	return self.vol.GetValue()

class RadioWindow(wx.Frame):
    def __init__(self,parent,id,title):
        wx.Frame.__init__(self,parent,wx.ID_ANY, title)
	# Need this for Windows
	self.SetBackgroundColour((220, 218, 213, 255))

	# GUI components
        #self.control = wx.TextCtrl(self, 1, style=wx.TE_MULTILINE)
        #self.CreateStatusBar() # A Statusbar in the bottom of the window

        # Menu bar
        filemenu= wx.Menu()
        filemenu.Append(ID_EXIT,"E&xit"," Terminate the program")
        # Creating the menubar.
        menuBar = wx.MenuBar()
        menuBar.Append(filemenu,"&File") # Adding the "filemenu" to the MenuBar
        self.SetMenuBar(menuBar)  # Adding the MenuBar to the Frame content.
        wx.EVT_MENU(self, ID_EXIT, self.OnExit)

	# Presets
	self.channels = Presets(self)

	# Frequency display
	#self.freqDial = FrequencyDial(self, -1, "Frequency", "100.000.0")
	self.freqDial = FrequencyDial(self)
	self.SetFrequency("88.0")

	# Keyboard
	self.keyboard = KeyBoard(self)

	# Mode selection
	self.modes = ("FM", "AM")
	self.currentmode = wx.RadioBox(
                       self, -1, "Mode", wx.DefaultPosition, (80,80),
                       self.modes, 1, wx.RA_SPECIFY_COLS
                       )
	self.Bind(wx.EVT_RADIOBOX, self.OnModeSelect)

	# Volume slider
	self.volume = VolumeSlider(self)

	# Main layout
	self.sizer = wx.GridBagSizer(10,10)
	self.sizer.Add(self.freqDial, (0,0), (3,6), wx.EXPAND)
	self.sizer.Add(self.keyboard, (3,0), (2,3), wx.EXPAND)
	self.sizer.Add(self.currentmode, (0,6), (3,1), wx.EXPAND)
	self.sizer.Add(self.channels, (3,3), (1,4), wx.EXPAND)
	self.sizer.Add(self.volume, (4,3), (1,4), wx.EXPAND)
        #Layout sizers
        self.SetSizer(self.sizer)
        self.SetAutoLayout(1)
        self.sizer.Fit(self)
        self.Show(1)

    def OnExit(self,e):
        self.Close(True)  # Close the frame.

    def OnModeSelect(self,e):
	m = self.currentmode.GetSelection()
	if m == msp3417.MODE_FM:
	    msp3417Dev.WriteDemRegister(0x0083, 0x0000)
	    print "Mode FM"
	elif m == msp3417.MODE_AM:
	    msp3417Dev.WriteDemRegister(0x0083, 0x816e)
	    print "Mode AM"

    def OnVolumeSet(self,e):
	v = self.volume.GetVolume()
	#msp3417Dev.WriteDspRegister(0x0007, (int(v*127/100) << 8) | 1)
	msp3417Dev.SetVolume(v)

    def OnPresetSelect(self, e):
	index = self.channels.GetPreset()
	self.SetFrequency(presets[index][1])
	self.SetMode(presets[index][2])

    def IgnoreKeyboard(self):
	self.freqDial.SetFrequency(self.f)

    def UpdateDisplay(self, f):
	self.freqDial.SetText("%s" % f)
	self.channels.ClearPreset()

    def SetFrequency(self, f):
	# Input in MHz
	if f == "" :
	   return
	f = int(float(f) * 1000000)
	self.f = f
	cxa3235.SetFrequency(f/1000)
	self.freqDial.SetFrequency(f)

    def OnCmd0(self):
	print "OnCmd0"
	print "QPD",msp3417Dev.ReadDspRegister(0x0019),msp3417Dev.ReadDspRegister(0x001a)

    def OnCmd1(self):
	print "OnCmd1"
	self.f += 50000
        cxa3235.SetFrequency(self.f/1000)
        self.freqDial.SetFrequency(self.f)
	print "QPD",msp3417Dev.ReadDspRegister(0x0019),msp3417Dev.ReadDspRegister(0x001a)

    def OnCmd2(self):
	print "OnCmd1"
	self.f -= 50000
        cxa3235.SetFrequency(self.f/1000)
        self.freqDial.SetFrequency(self.f)
	print "QPD",msp3417Dev.ReadDspRegister(0x0019),msp3417Dev.ReadDspRegister(0x001a)

    def SetMode(self, m):
	#if m == MODE_FM:
	#    msp3417Dev.WriteDemRegister(0x0083, 0x0000)
	#    print "Mode FM"
	#elif m == MODE_AM:
	#    msp3417Dev.WriteDemRegister(0x0083, 0x816e)
	#    print "Mode AM"
	msp3417Dev.SetMode(m)
	self.currentmode.SetSelection(m)

    def SetVolume(self,v):
	msp3417Dev.WriteDspRegister(0x0007, (int(v*127/100) << 8) | 1)

# Main

TA1243_ADDR = 0x65
CXA3235_ADDR = 0x60
MSP3417_ADDR = 0x40

i2cTinyUsbDev = i2c_tiny_usb.I2cTinyUsbDevice()
handle = i2cTinyUsbDev.open()
i2cTinyUsbDev.setDelay(10)

#ta1243 = ta1243.Ta1243Device(i2cTinyUsbDev, TA1243_ADDR)
cxa3235 = cxa3235.Cxa3235Device(i2cTinyUsbDev, CXA3235_ADDR)
msp3417Dev = msp3417.Msp3417Device(i2cTinyUsbDev, MSP3417_ADDR)

app = wx.PySimpleApp()
frame = RadioWindow(None, -1, "VHF/UHF radio")
app.MainLoop()

